{*******************************************************************}
{* 2x2-bits-multiplier simulation unit for LogicGates7 application *}
{*******************************************************************}

unit binmult22;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, common;

type
  {*************}
  { TfBinMult22 }
  {*************}
  TfBinMult22 = class(TForm)
    StaticText1: TStaticText;
    Label1, Label10, Label18: TLabel;
    imSwitchA0, imSwitchA1, imSwitchB0, imSwitchB1: TImage;
    shSwitchA0a, shSwitchA0b, shSwitchA1a, shSwitchA1b: TShape;
    shSwitchB0a, shSwitchB0b, shSwitchB1a, shSwitchB1b: TShape;
    shLedA0, shLedA1, shLedB0, shLedB1: TShape;
    shLedC0, shLedC1, shLedC2, shLedC3: TShape;
    laA0, laA1, laB0, laB1, laC0, laC1, laC2, laC3: TLabel;
    Image1, Image2, Image3, Image4, Image5, Image6: TImage;
    Shape1, Shape21, Shape22, Shape23, Shape25, Shape28, Shape29, Shape32, Shape35: TShape;
    Shape52, Shape53, Shape54, Shape55, Shape56, Shape57, Shape58, Shape59: TShape;
    Shape60, Shape61, Shape62, Shape64, Shape65, Shape66, Shape67, Shape68: TShape;
    Shape63, Shape69, Shape70, Shape72, Shape73, Shape74, Shape75: TShape;
    Shape71, Shape76, Shape77, shape78, Shape79, Shape80, Shape81, Shape83: TShape;
    Shape85, Shape86, Shape87, Shape88, Shape89, Shape90, Shape91, Shape92: TShape;
    Shape93, Shape95, Shape96, Shape97, Shape99, Shape82, Shape84, Shape94, Shape98: TShape;
    Shape100, Shape101, Shape102, Shape103, Shape104, Shape105, Shape106, Shape107: TShape;
    Shape108, Shape109, Shape110, Shape111, Shape116, Shape122, Shape123, Shape124: TShape;
    sgMult: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchA0Click(Sender: TObject);
    procedure imSwitchA1Click(Sender: TObject);
    procedure imSwitchB0Click(Sender: TObject);
    procedure imSwitchB1Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aLeds: TLeds;
    aSwitches: TSwitches;
    bSwitches, bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape)
  DSwitches: array[0..3] of TSwitchData = (
    (Name: 'SwitchA0';
     Data: (246, 268, 248)),
    (Name: 'SwitchA1';
     Data: (53, 75, 55)),
    (Name: 'SwitchB0';
     Data: (344, 366, 346)),
    (Name: 'SwitchB1';
     Data: (151, 173, 153))
  );

var
  fBinMult22: TfBinMult22;

implementation

{$R *.lfm}

{ 2x2-bits multiplier: Set switch position; do binary multiplication; turn LEDs on/off; fill-in multiplication-table }

procedure SwitchAndMult(DSwitches: array of TSwitchData; var Switches: TSwitches; var SwitchStatus: TStatus;
  var Leds: TLeds; var LedStatus: TStatus; IX: Integer);

const
  Bits: array[False..True] of Integer = (0, 1);
  HexDigits: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' );

var
  Exp, Hex, I, J: Integer;
  C0, C1, C2, C3: Boolean;
  Dec: array[0..2] of Integer;

begin
  // Change current input from 0 to +Vs (and vice-versa)
  SwitchStatus[IX] := not SwitchStatus[IX];
  // Corresponding LED status
  LedStatus[IX] := SwitchStatus[IX];
  // Set switch positions
  SwitchesOnOff(DSwitches, Switches, SwitchStatus);
  // Do the multiplication
  Multiplier2x2(LedStatus[0], LedStatus[1], LedStatus[2], LedStatus[3], C0, C1, C2, C3);
  LedStatus[4] := C0; LedStatus[5] := C1; LedStatus[6] := C2; LedStatus[7] := C3;
  // Turn all LEDs on/off
  LedsOnOff(Leds, LedStatus);
  // Fill-in binary values and calculate decimal values
  Dec[0] := 0; Dec[1] := 0; Dec[2] := 0; Exp := 1;
  for I := 1 to 4 do begin
    for J := 1 to 3 do
      fBinMult22.sgMult.Cells[I, J] := ' 0';
  end;
  for I := 0 to 3 do begin
    if I <= 1 then begin
      fBinMult22.sgMult.Cells[4 - I, 1] := ' ' + IntToStr(Bits[LedStatus[I]]);
      fBinMult22.sgMult.Cells[4 - I, 2] := ' ' + IntToStr(Bits[LedStatus[I + 2]]);
    end;
    fBinMult22.sgMult.Cells[4 - I, 3] := ' ' + IntToStr(Bits[LedStatus[I + 4]]);
    if I <= 1 then begin
      Dec[0] += Bits[LedStatus[I]] * Exp; Dec[1] += Bits[LedStatus[I + 2]] * Exp;
    end;
    Dec[2] += Bits[LedStatus[I + 4]] * Exp;
    Exp *= 2;
  end;
  // Fill-in decimal and (here calculated) hexadecimal values
  for I := 0 to 2 do begin
    fBinMult22.sgMult.Cells[6, I + 1] := '  ' + IntToStr(Dec[I]);
    if Dec[I] < 10 then
      fBinMult22.sgMult.Cells[6, I + 1] := ' ' + fBinMult22.sgMult.Cells[6, I + 1];
    Hex := Dec[I] mod 16;
    fBinMult22.sgMult.Cells[5, I + 1] := '  ' + HexDigits[Hex];
  end;
end;

{*************}
{ TfBinMult22 }
{*************}

{ Application start: Initialisation }

procedure TfBinMult22.FormCreate(Sender: TObject);

begin
  // Arrays with switches (transparent image and bar shapes) and their "on/off" status
  SetLength(aSwitches, 4); SetLength(bSwitches, 4);
  aSwitches[0].Switch0 := imSwitchA0; aSwitches[0].Switch1 := shSwitchA0a; aSwitches[0].Switch2 := shSwitchA0b;
  aSwitches[1].Switch0 := imSwitchA1; aSwitches[1].Switch1 := shSwitchA1a; aSwitches[1].Switch2 := shSwitchA1b;
  aSwitches[2].Switch0 := imSwitchB0; aSwitches[2].Switch1 := shSwitchB0a; aSwitches[2].Switch2 := shSwitchB0b;
  aSwitches[3].Switch0 := imSwitchB1; aSwitches[3].Switch1 := shSwitchB1a; aSwitches[3].Switch2 := shSwitchB1b;
  // Arrays with LEDs (shape) and their "on/off" status
  SetLength(aLeds, 8); SetLength(bLeds, 8);
  aLeds[0] := shLedA0; aLeds[1] := shLedA1; aLeds[2] := shLedB0; aLeds[3] := shLedB1;
  aLeds[4] := shLedC0; aLeds[5] := shLedC1; aLeds[6] := shLedC2; aLeds[7] := shLedC3;
  // Use subscripts in input/output labels
  laA0.Caption := 'A' + SUB_Digits[0]; laA1.Caption := 'A' + SUB_Digits[1];
  laB0.Caption := 'B' + SUB_Digits[0]; laB1.Caption := 'B' + SUB_Digits[1];
  laC0.Caption := 'C' + SUB_Digits[0]; laC1.Caption := 'C' + SUB_Digits[1];
  laC2.Caption := 'C' + SUB_Digits[2]; laC3.Caption := 'C' + SUB_Digits[3];
end;

{ Application show-up: Initialize 2x2-bit multiplier simulation }

procedure TfBinMult22.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All switches (inputs) to 0
  for I := 0 to Length(bSwitches) - 1 do
    bSwitches[I] := False;
  SwitchesOnOff(DSwitches, aSwitches, bSwitches);
  // All LEDs off
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A0, A1, B0, B1" activated: Do 2x2-bits multiplier simulation for actual switch positions }

procedure TfBinMult22.imSwitchA0Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 0);
end;

procedure TfBinMult22.imSwitchA1Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 1);
end;

procedure TfBinMult22.imSwitchB0Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 2);
end;

procedure TfBinMult22.imSwitchB1Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 3);
end;

{ Button "Close": Close the window }

procedure TfBinMult22.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

