{*******************************************************************}
{* 4x2-bits-multiplier simulation unit for LogicGates7 application *}
{*******************************************************************}

unit binmult42;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, common;

type
  {*************}
  { TfBinMult42 }
  {*************}
  TfBinMult42 = class(TForm)
    StaticText1: TStaticText;
    Label1, Label2, Label10, Label11, Label18: TLabel;
    imSwitchA0, imSwitchA1, imSwitchB0, imSwitchB1, imSwitchB2, imSwitchB3: TImage;
    shSwitchA0a, shSwitchA0b, shSwitchA1a, shSwitchA1b: TShape;
    shSwitchB0a, shSwitchB0b, shSwitchB1a, shSwitchB1b: TShape;
    shSwitchB2a, shSwitchB2b, shSwitchB3a, shSwitchB3b: TShape;
    shLedA0, shLedA1, shLedB0, shLedB1, shLedB2, shLedB3: TShape;
    shLedC0, shLedC1, shLedC2, shLedC3, shLedC4, shLedC5: TShape;
    laA0, laA1, laB0, laB1, laB2, laB3, laC0, laC1, laC2, laC3, laC4, laC5: TLabel;
    Image1, Image2, Image3, Image4, Image5, Image6, Image7, Image8, Image9: TImage;
    Shape1, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26, Shape27, Shape28: TShape;
    Shape30, Shape32, Shape33, Shape36, Shape55, Shape58, Shape78, Shape81, Shape84: TShape;
    Shape85, Shape86, Shape87, Shape88, Shape89, Shape90, Shape91, Shape92, Shape93: TShape;
    Shape94, Shape95, Shape96, Shape97, Shape98, Shape99, Shape100, Shape101, Shape102: TShape;
    Shape103, Shape104, Shape105, Shape106, Shape107, Shape108, Shape109, Shape110, Shape111: TShape;
    Shape112, Shape113, Shape114, Shape115, Shape116, Shape117, Shape118, Shape119, Shape120: TShape;
    Shape121, Shape122, Shape123, Shape124, Shape125, Shape126, Shape127, Shape128, Shape129: TShape;
    Shape130, Shape131, Shape132, Shape133, Shape134, Shape135, Shape136, Shape137, Shape138: TShape;
    Shape139, Shape140, Shape141, Shape142, Shape143, Shape144, Shape145, Shape146, Shape147: TShape;
    Shape148, Shape149, Shape150, Shape151, Shape152, Shape153, Shape154, Shape155, Shape156: TShape;
    Shape157, Shape158, Shape159, Shape160, Shape161, Shape162, Shape163, Shape164, Shape165: TShape;
    Shape166, Shape167, Shape168, Shape169, Shape170, Shape171: TShape;
    sgMult: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchA0Click(Sender: TObject);
    procedure imSwitchA1Click(Sender: TObject);
    procedure imSwitchB0Click(Sender: TObject);
    procedure imSwitchB1Click(Sender: TObject);
    procedure imSwitchB2Click(Sender: TObject);
    procedure imSwitchB3Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aLeds: TLeds;
    aSwitches: TSwitches;
    bSwitches, bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape)
  DSwitches: array[0..5] of TSwitchData = (
    (Name: 'SwitchA0';
     Data: (513, 535, 515)),
    (Name: 'SwitchA1';
     Data: (280, 302, 282)),
    (Name: 'SwitchB0';
     Data: (625, 647, 627)),
    (Name: 'SwitchB1';
     Data: (398, 420, 400)),
    (Name: 'SwitchB2';
     Data: (166, 188, 168)),
    (Name: 'SwitchB3';
     Data: (53, 75, 55))
  );

var
  fBinMult42: TfBinMult42;

implementation

{$R *.lfm}

{ 4x2-bits multiplier: Set switch position; do binary multiplication; turn LEDs on/off; fill-in multiplication-table }

procedure SwitchAndMult(DSwitches: array of TSwitchData; var Switches: TSwitches; var SwitchStatus: TStatus;
  var Leds: TLeds; var LedStatus: TStatus; IX: Integer);

const
  Bits: array[False..True] of Integer = (0, 1);
  HexDigits: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' );

var
  Exp, Hex1, Hex2, I, J: Integer;
  Dec: array[0..2] of Integer;
  InputA: array[0..1] of Boolean;
  InputB: array[0..3] of Boolean;
  Output: TBooleans;

begin
  // Change current input from 0 to +Vs (and vice-versa)
  SwitchStatus[IX] := not SwitchStatus[IX];
  // Corresponding LED status
  LedStatus[IX] := SwitchStatus[IX];
  // Set switch positions
  SwitchesOnOff(DSwitches, Switches, SwitchStatus);
  // Do the multiplication
  for I := 0 to 1 do
    InputA[I] := LedStatus[I];
  for I := 0 to 3 do
    InputB[I] := LedStatus[I + 2];
  Multiplier4x2(InputA, InputB, Output);
  for I := 0 to 5 do
    LedStatus[I + 6] := Output[I];
  // Turn all LEDs on/off
  LedsOnOff(Leds, LedStatus);
  // Fill-in binary values and calculate decimal values
  Dec[0] := 0; Dec[1] := 0; Dec[2] := 0; Exp := 1;
  for I := 1 to 6 do begin
    for J := 1 to 3 do
      fBinMult42.sgMult.Cells[I, J] := ' 0';
  end;
  for I := 0 to 5 do begin
    if I <= 1 then
      fBinMult42.sgMult.Cells[6 - I, 1] := ' ' + IntToStr(Bits[LedStatus[I]]);
    if I <= 3 then
      fBinMult42.sgMult.Cells[6 - I, 2] := ' ' + IntToStr(Bits[LedStatus[I + 2]]);
    fBinMult42.sgMult.Cells[6 - I, 3] := ' ' + IntToStr(Bits[LedStatus[I + 6]]);
    if I <= 1 then
      Dec[0] += Bits[LedStatus[I]] * Exp;
    if I <= 3 then
      Dec[1] += Bits[LedStatus[I + 2]] * Exp;
    Dec[2] += Bits[LedStatus[I + 6]] * Exp;
    Exp *= 2;
  end;
  // Fill-in decimal and (here calculated) hexadecimal values
  for I := 0 to 2 do begin
    fBinMult42.sgMult.Cells[8, I + 1] := '  ' + IntToStr(Dec[I]);
    if Dec[I] < 10 then
      fBinMult42.sgMult.Cells[8, I + 1] := ' ' + fBinMult42.sgMult.Cells[8, I + 1];
    Hex1 := Dec[I] div 16; Hex2 := Dec[I] mod 16;
    fBinMult42.sgMult.Cells[7, I + 1] := '  ' + HexDigits[Hex1] + HexDigits[Hex2];
  end;
end;

{*************}
{ TfBinMult42 }
{*************}

{ Application start: Initialisation }

procedure TfBinMult42.FormCreate(Sender: TObject);

begin
  // Arrays with switches (transparent image and bar shapes) and their "on/off" status
  SetLength(aSwitches, 6); SetLength(bSwitches, 6);
  aSwitches[0].Switch0 := imSwitchA0; aSwitches[0].Switch1 := shSwitchA0a; aSwitches[0].Switch2 := shSwitchA0b;
  aSwitches[1].Switch0 := imSwitchA1; aSwitches[1].Switch1 := shSwitchA1a; aSwitches[1].Switch2 := shSwitchA1b;
  aSwitches[2].Switch0 := imSwitchB0; aSwitches[2].Switch1 := shSwitchB0a; aSwitches[2].Switch2 := shSwitchB0b;
  aSwitches[3].Switch0 := imSwitchB1; aSwitches[3].Switch1 := shSwitchB1a; aSwitches[3].Switch2 := shSwitchB1b;
  aSwitches[4].Switch0 := imSwitchB2; aSwitches[4].Switch1 := shSwitchB2a; aSwitches[4].Switch2 := shSwitchB2b;
  aSwitches[5].Switch0 := imSwitchB3; aSwitches[5].Switch1 := shSwitchB3a; aSwitches[5].Switch2 := shSwitchB3b;
  // Arrays with LEDs (shape) and their "on/off" status
  SetLength(aLeds, 12); SetLength(bLeds, 12);
  aLeds[0] := shLedA0; aLeds[1] := shLedA1; aLeds[2] := shLedB0; aLeds[3] := shLedB1; aLeds[4] := shLedB2; aLeds[5] := shLedB3;
  aLeds[6] := shLedC0; aLeds[7] := shLedC1; aLeds[8] := shLedC2; aLeds[9] := shLedC3; aLeds[10] := shLedC4; aLeds[11] := shLedC5;
  // Use subscripts in input/output labels
  laA0.Caption := 'A' + SUB_Digits[0]; laA1.Caption := 'A' + SUB_Digits[1];
  laB0.Caption := 'B' + SUB_Digits[0]; laB1.Caption := 'B' + SUB_Digits[1];
  laB2.Caption := 'B' + SUB_Digits[2]; laB3.Caption := 'B' + SUB_Digits[3];
  laC0.Caption := 'C' + SUB_Digits[0]; laC1.Caption := 'C' + SUB_Digits[1];
  laC2.Caption := 'C' + SUB_Digits[2]; laC3.Caption := 'C' + SUB_Digits[3];
  laC4.Caption := 'C' + SUB_Digits[4]; laC5.Caption := 'C' + SUB_Digits[5];
end;

{ Application show-up: Initialize 4x2-bit multiplier simulation }

procedure TfBinMult42.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All switches (inputs) to 0
  for I := 0 to Length(bSwitches) - 1 do
    bSwitches[I] := False;
  SwitchesOnOff(DSwitches, aSwitches, bSwitches);
  // All LEDs off
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A0, A1, B0, B1, B2, B3" activated: Do 4x2-bits multiplier simulation for actual switch positions }

procedure TfBinMult42.imSwitchA0Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 0);
end;

procedure TfBinMult42.imSwitchA1Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 1);
end;

procedure TfBinMult42.imSwitchB0Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 2);
end;

procedure TfBinMult42.imSwitchB1Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 3);
end;

procedure TfBinMult42.imSwitchB2Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 4);
end;

procedure TfBinMult42.imSwitchB3Click(Sender: TObject);

begin
  SwitchAndMult(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 5);
end;

procedure TfBinMult42.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

